@@ -9,7 +9,7 @@ use Carp;
 
 use constant HOSTNAME => hostname();
 
-$Net::Server::Mail::VERSION = '0.20';
+$Net::Server::Mail::VERSION = '0.22';
 
 =pod
 
@@ -89,7 +89,7 @@ ESMTP extensions. Developers can hook code at each SMTP session state
 and change the module's behaviors by registering event call-backs. The
 class is designed to be easily inherited from.
 
-This class is the base class for mail service protocols such axs
+This class is the base class for mail service protocols such as
 B<Net::Server::Mail::SMTP>, B<Net::Server::Mail::ESMTP> and
 B<Net::Server::Mail::LMTP>. Refer to the documentation provided with
 each of these modules.
@@ -134,62 +134,51 @@ A value of 0 turns this feature off. Defaults to 0.
 
 =cut
 
-sub new
-{
-    my($proto, @args) = @_;
+sub new {
+    my ( $proto, @args ) = @_;
     my $class = ref $proto || $proto;
-    my $self  = {};
-    bless($self, $class);
+    my $self = {};
+    bless( $self, $class );
     return $self->init(@args);
 }
 
-sub init
-{
+sub init {
     my $self = shift;
-    confess("odd number of arguments") if(@_ % 2);
-    my $options = $self->{options} =
-    {
-        handle_in           => undef,
-        handle_out          => undef,
-        socket              => undef,
-        error_sleep_time    => 0,
-        idle_timeout        => 0,
+    confess("odd number of arguments") if ( @_ % 2 );
+    my $options = $self->{options} = {
+        handle_in        => undef,
+        handle_out       => undef,
+        socket           => undef,
+        error_sleep_time => 0,
+        idle_timeout     => 0,
     };
-    for(my $i = 0; $i < @_; $i += 2)
-    {
-        $options->{lc($_[$i])} = $_[$i + 1];
+    for ( my $i = 0 ; $i < @_ ; $i += 2 ) {
+        $options->{ lc( $_[$i] ) } = $_[ $i + 1 ];
     }
 
-    if(defined $options->{handle_in} && defined $options->{handle_out})
-    {
-        if(UNIVERSAL::isa($options->{handle_in},'IO::Handle'))
-        {
+    if ( defined $options->{handle_in} && defined $options->{handle_out} ) {
+        if ( UNIVERSAL::isa( $options->{handle_in}, 'IO::Handle' ) ) {
             $self->{in} = $options->{handle_in};
         }
-        else
-        {
-            $self->{in} = 
-              IO::Handle->new->fdopen(fileno($options->{handle_in}), "r");
+        else {
+            $self->{in} =
+              IO::Handle->new->fdopen( fileno( $options->{handle_in} ), "r" );
         }
-        if(UNIVERSAL::isa($options->{handle_out},'IO::Handle'))
-        {
+        if ( UNIVERSAL::isa( $options->{handle_out}, 'IO::Handle' ) ) {
             $self->{out} = $options->{handle_out};
         }
-        else
-        {
-            $self->{out} = 
-              IO::Handle->new->fdopen(fileno($options->{handle_out}), "w");
+        else {
+            $self->{out} =
+              IO::Handle->new->fdopen( fileno( $options->{handle_out} ), "w" );
         }
     }
-    elsif(defined $options->{'socket'})
-    {
+    elsif ( defined $options->{'socket'} ) {
         $self->{in}  = $options->{'socket'};
         $self->{out} = $options->{'socket'};
     }
-    else
-    {
-        $self->{in}  = IO::Handle->new->fdopen(fileno(STDIN), "r");
-        $self->{out} = IO::Handle->new->fdopen(fileno(STDOUT), "w");
+    else {
+        $self->{in}  = IO::Handle->new->fdopen( fileno(STDIN),  "r" );
+        $self->{out} = IO::Handle->new->fdopen( fileno(STDOUT), "w" );
     }
 
     $self->{out}->autoflush(1);
@@ -202,152 +191,131 @@ sub init
 
 =head2 dojob
 
-Some command need to do some job after the handler call. Handler may
-want to overide this comportement to prevent from this job being
-executed.
+Some commands need to do a job after the handler call. The handler
+may want to override this to prevent the job from being executed.
 
-By calling this method with a (defined) false value as argument,
-expected job isn't executed. Defaults to true.
+By calling this method with a (defined) false value as an argument,
+the expected job isn't executed. Defaults to true.
 
 =cut
 
-sub init_dojob {shift->{_dojob} = 1;}
-sub dojob
-{
-    my($self, $bool) = @_;
-    $self->{_dojob} = $bool if(defined $bool);
+sub init_dojob { shift->{_dojob} = 1; }
+
+sub dojob {
+    my ( $self, $bool ) = @_;
+    $self->{_dojob} = $bool if ( defined $bool );
     return $self->{_dojob};
 }
 
-sub make_event
-{
+sub make_event {
     my $self = shift;
-    confess('odd number of arguments') if(@_ % 2);
+    confess('odd number of arguments') if ( @_ % 2 );
     my %args = @_;
 
     my $name = $args{'name'} || confess('missing argument: \'name\'');
-    my $args = defined $args{'arguments'} && ref $args{'arguments'} eq 'ARRAY'
-        ? $args{'arguments'} : [];
+    my $args =
+      defined $args{'arguments'}
+      && ref $args{'arguments'} eq 'ARRAY' ? $args{'arguments'} : [];
 
     $self->init_dojob();
-    my($success, $code, $msg) = $self->callback($name, @{$args});
+    my ( $success, $code, $msg ) = $self->callback( $name, @{$args} );
 
     # we have to take a proper decision if successness is undefined
-    if(not defined $success)
-    {
-        if(exists $args{'default_reply'})
-        {
-            if(ref $args{'default_reply'} eq 'ARRAY')
-            {
-                ($success, $code, $msg) = $args{'default_reply'};
+    if ( not defined $success ) {
+        if ( exists $args{'default_reply'} ) {
+            if ( ref $args{'default_reply'} eq 'ARRAY' ) {
+                ( $success, $code, $msg ) = $args{'default_reply'};
                 $success = 0 unless defined $success;
             }
-            else
-            {
+            else {
                 $success = $args{'default_reply'};
             }
         }
-        else
-        {
-            $success = 1; # default
+        else {
+            $success = 1;    # default
         }
     }
 
     # command may have some job to do regarding to the result. handler
     # can avoid it by calling dojob() method with a false value.
-    if($self->dojob())
-    {
-        if($success)
-        {
-            if(defined $args{'on_success'}
-               and ref $args{'on_success'} eq 'CODE')
+    if ( $self->dojob() ) {
+        if ($success) {
+            if ( defined $args{'on_success'}
+                and ref $args{'on_success'} eq 'CODE' )
             {
-                &{$args{'on_success'}};
+                &{ $args{'on_success'} };
             }
         }
-        else
-        {
-            if(defined $args{'on_failure'}
-               and ref $args{'on_failure'} eq 'CODE')
+        else {
+            if ( defined $args{'on_failure'}
+                and ref $args{'on_failure'} eq 'CODE' )
             {
-                &{$args{'on_failure'}};
+                &{ $args{'on_failure'} };
             }
         }
     }
 
     # ensure that a reply is sent, all SMTP command need at most 1 reply.
-    unless(defined $code)
-    {
-        if(defined $success && $success)
-        {
-            ($code, $msg) =
-              $self->get_default_reply($args{'success_reply'}, 250);
+    # some events such as 'stop_session' don't require sending reply.
+    unless ( defined $code && !$args{'no_reply'} ) {
+        if ( defined $success && $success ) {
+            ( $code, $msg ) =
+              $self->get_default_reply( $args{'success_reply'}, 250 );
         }
-        else
-        {
-            ($code, $msg) =
-              $self->get_default_reply($args{'failure_reply'}, 550);
+        else {
+            ( $code, $msg ) =
+              $self->get_default_reply( $args{'failure_reply'}, 550 );
         }
     }
 
-    die "return code `$code' isn't numeric" if($code =~ /\D/);
+    die "return code `$code' isn't numeric" if ( defined $code && $code =~ /\D/ );
 
-    $self->handle_reply($name, $success, $code, $msg)
+    $self->handle_reply( $name, $success, $code, $msg )
       if defined $code and length $code;
 
     return $success;
 }
 
-sub get_default_reply
-{
-    my($self, $config, $default) = @_;
+sub get_default_reply {
+    my ( $self, $config, $default ) = @_;
 
-    my($code, $msg);
-    if(defined $config)
-    {
-        if(ref $config eq 'ARRAY')
-        {
-            ($code, $msg) = @$config;
+    my ( $code, $msg );
+    if ( defined $config ) {
+        if ( ref $config eq 'ARRAY' ) {
+            ( $code, $msg ) = @$config;
         }
-        elsif(not ref $config)
-        {
+        elsif ( not ref $config ) {
             $code = $config;
         }
-        else
-        {
+        else {
             confess("unexpected format for reply");
         }
     }
-    else
-    {
+    else {
         $code = $default;
     }
 
-    return($code, $msg);
+    return ( $code, $msg );
 }
 
-sub handle_reply
-{
-    my($self, $verb, $success, $code, $msg) = @_;
+sub handle_reply {
+    my ( $self, $verb, $success, $code, $msg ) = @_;
+
     # don't reply anything if code is empty
-    $self->reply($code, $msg) if(length $code);
+    $self->reply( $code, $msg ) if ( length $code );
 }
 
-sub callback
-{
-    my($self, $name, @args) = @_;
+sub callback {
+    my ( $self, $name, @args ) = @_;
 
-    if(defined $self->{callback}->{$name})
-    {
+    if ( defined $self->{callback}->{$name} ) {
         my @rv;
-        eval
-        {
-            my($code, $context) = @{$self->{callback}->{$name}};
+        eval {
+            my ( $code, $context ) = @{ $self->{callback}->{$name} };
             $self->set_context($context);
-            @rv = &{$code}($self, @args);
+            @rv = &{$code}( $self, @args );
         };
-        if($@)
-        {
+        if ($@) {
             confess $@;
         }
         return @rv;
@@ -356,15 +324,13 @@ sub callback
     return 1;
 }
 
-sub set_context
-{
-    my($self, $context) = @_;
+sub set_context {
+    my ( $self, $context ) = @_;
     $self->{_context} = $context;
 }
 
-sub get_context
-{
-    my($self) = @_;
+sub get_context {
+    my ($self) = @_;
     return $self->{_context};
 }
 
@@ -396,51 +362,45 @@ return 1 to 3 values: (success, [return_code, ["message"]]).
 
 =cut
 
-sub set_callback
-{
-    my($self, $name, $code, $context) = @_;
+sub set_callback {
+    my ( $self, $name, $code, $context ) = @_;
     confess('bad callback() invocation')
-        unless defined $code && ref $code eq 'CODE';
-    $self->{callback}->{$name} = [$code, $context];
+      unless defined $code && ref $code eq 'CODE';
+    $self->{callback}->{$name} = [ $code, $context ];
 }
 
-sub def_verb
-{
-    my($self, $verb, $coderef) = @_;
-    $self->{verb}->{uc $verb} = $coderef;
+sub def_verb {
+    my ( $self, $verb, $coderef ) = @_;
+    $self->{verb}->{ uc $verb } = $coderef;
 }
 
-sub undef_verb
-{
-    my($self, $verb) = @_;
+sub undef_verb {
+    my ( $self, $verb ) = @_;
     delete $self->{verb}->{$verb}
-        if defined $self->{verb};
+      if defined $self->{verb};
 }
 
-sub list_verb
-{
-    my($self) = @_;
-    return keys %{$self->{verb}};
+sub list_verb {
+    my ($self) = @_;
+    return keys %{ $self->{verb} };
 }
 
-
-sub next_input_to
-{
-    my($self, $method_ref) = @_;
+sub next_input_to {
+    my ( $self, $method_ref ) = @_;
     $self->{next_input} = $method_ref
-      if(defined $method_ref);
-    return $self->{next_input}
+      if ( defined $method_ref );
+    return $self->{next_input};
 }
 
-sub tell_next_input_method
-{
+sub tell_next_input_method {
+
+    my ( $self, $input ) = @_;
 
-    my($self, $input) = @_;
     # calling the method and reinitialize. Note: we have to reinit
     # before calling the code, because code can resetup this variable.
     my $code = $self->{next_input};
     undef $self->{next_input};
-    my $rv = &{$code}($self, $input);
+    my $rv = &{$code}( $self, $input );
     return $rv;
 }
 
@@ -454,179 +414,174 @@ Start a new session.
 
 =cut
 
-sub process
-{
-    my($self) = @_;
+sub process {
+    my ($self) = @_;
 
     my $in  = $self->{in};
     my $sel = new IO::Select;
     $sel->add($in);
 
     $self->banner;
+
     # switch to non-blocking socket to handle PIPELINING
     # ESMTP extension. See RFC 2920 for more details.
-    if($^O eq 'MSWin32')
-    {
+    if ( $^O eq 'MSWin32' ) {
+
         # win32 platforms don't support nonblocking IO
-        ioctl($in, 2147772030, 1);
+        ioctl( $in, 2147772030, 1 );
     }
-    else
-    {
-        defined($in->blocking(0)) or die "Couldn't set nonblocking: $^E";
+    else {
+        defined( $in->blocking(0) ) or die "Couldn't set nonblocking: $^E";
     }
-    
-    my $buffer = ""; 
+
+    my $buffer = "";
     while (1) {
+
         # wait for data and read it
         my $rv = undef;
 
-        if ($sel->can_read($self->{options}->{idle_timeout} || undef))
-        {
-            if ($^O eq 'MSWin32')
-            {
+        if ( $sel->can_read( $self->{options}->{idle_timeout} || undef ) ) {
+            if ( $^O eq 'MSWin32' ) {
+
                 # see how much data is available to read
-                my $size = pack("L",0);
-                ioctl($in, 1074030207, $size);
-                $size = unpack("L", $size);
+                my $size = pack( "L", 0 );
+                ioctl( $in, 1074030207, $size );
+                $size = unpack( "L", $size );
 
                 # read the data to $buffer
-                $rv = sysread($in, $buffer, $size, length($buffer));
+                $rv = sysread( $in, $buffer, $size, length($buffer) );
             }
-            else
-            {
-                $rv = sysread($in, $buffer, 512*1024, length($buffer));
+            else {
+                $rv = sysread( $in, $buffer, 512 * 1024, length($buffer) );
             }
         }
-        if ((not defined $rv) or ($rv == 0)) {
-            # timeout, read error or connection closed
-            last;
+	else {
+	    # timeout
+	    return $self->timeout;
+	}
+
+        if ( ( not defined $rv ) or ( $rv == 0 ) ) {
+
+            # read error or connection closed
+            return $self->stop_session((not defined $rv) ? ($!) : ());
         }
 
         # process all terminated lines
         # Note: Should accept only CRLF according to RFC. We accept
         # plain LFs anyway because its more liberal and works as well.
-        my $newline_idx = rindex($buffer, "\n");
-        if ($newline_idx >= 0) {
+        my $newline_idx = rindex( $buffer, "\n" );
+        if ( $newline_idx >= 0 ) {
+
             # one or more lines, terminated with \r?\n
-            my $chunk = substr($buffer, 0, $newline_idx+1);
+            my $chunk = substr( $buffer, 0, $newline_idx + 1 );
+
             # remaining buffer
-            $buffer = substr($buffer, $newline_idx+1);
+            $buffer = substr( $buffer, $newline_idx + 1 );
 
             my $rv;
-            if(defined $self->next_input_to())
-            {
+            if ( defined $self->next_input_to() ) {
                 $rv = $self->tell_next_input_method($chunk);
             }
-            else
-            {
-                $rv = $self->{process_operation}($self, $chunk);
+            else {
+                $rv = $self->{process_operation}( $self, $chunk );
             }
+
             # if $rv is defined, we have to close the connection
-            if (defined $rv) {
+            if ( defined $rv ) {
                 return $rv;
             }
         }
 
         # limit the size of lines to protect from excessive memory consumption
         # (RFC specifies 1000 bytes including \r\n)
-        if (length($buffer) > 1000) {
+        if ( length($buffer) > 1000 ) {
             $self->make_event(
-                name => 'linetobig',
-                success_reply => [552, 'line too long' ]);
+                name          => 'linetobig',
+                success_reply => [ 552, 'line too long' ]
+            );
             return 1;
         }
     }
 
-    $self->timeout;
+    return 1;
 }
 
-sub process_once
-{
-    my($self, $operation) = @_;
-    if($self->next_input_to())
-    {
+sub process_once {
+    my ( $self, $operation ) = @_;
+    if ( $self->next_input_to() ) {
         return $self->tell_next_input_method($operation);
     }
-    else
-    {
-        return $self->{process_operation}($self, $operation);
+    else {
+        return $self->{process_operation}( $self, $operation );
     }
 }
 
-sub process_operation
-{
-    my($self, $operation) = @_;
-    my($verb, $params) = $self->tokenize_command($operation);
-    if(defined $params && $params =~ /[\r\n]/)
-    {
+sub process_operation {
+    my ( $self, $operation ) = @_;
+    my ( $verb, $params )    = $self->tokenize_command($operation);
+    if ( defined $params && $params =~ /[\r\n]/ ) {
+
         # doesn't support grouping of operations
-        $self->reply(453, "Command received prior to completion of".
-                     " previous command sequence");
+        $self->reply( 453,
+                "Command received prior to completion of"
+              . " previous command sequence" );
         return;
     }
-    $self->process_command($verb, $params);
+    $self->process_command( $verb, $params );
 }
 
-sub process_command
-{
-    my($self, $verb, $params) = @_;
+sub process_command {
+    my ( $self, $verb, $params ) = @_;
 
-    if(exists $self->{verb}->{$verb})
-    {
+    if ( exists $self->{verb}->{$verb} ) {
         my $action = $self->{verb}->{$verb};
         my $rv;
-        if(ref $action eq 'CODE')
-        {
-            $rv = &{$self->{verb}->{$verb}}($self, $params);
+        if ( ref $action eq 'CODE' ) {
+            $rv = &{ $self->{verb}->{$verb} }( $self, $params );
         }
-        else
-        {
+        else {
             $rv = $self->$action($params);
         }
         return $rv;
     }
-    else
-    {
-        $self->reply(500, 'Syntax error: unrecognized command');
+    else {
+        $self->reply( 500, 'Syntax error: unrecognized command' );
         return;
     }
 }
 
-sub tokenize_command
-{
-    my($self, $line) = @_;
+sub tokenize_command {
+    my ( $self, $line ) = @_;
     $line =~ s/\r?\n$//s;
     $line =~ s/^\s+|\s+$//g;
-    my($verb, $params) = split ' ', $line, 2;
-    return(uc($verb), $params);
+    my ( $verb, $params ) = split ' ', $line, 2;
+    return ( uc($verb), $params );
 }
 
-sub reply
-{
-    my($self, $code, $msg) = @_;
+sub reply {
+    my ( $self, $code, $msg ) = @_;
     my $out = $self->{out};
+
     # tempo on error
     sleep $self->{options}->{error_sleep_time}
-        if($code >= 400 && $self->{options}->{error_sleep_time});
+      if ( $code >= 400 && $self->{options}->{error_sleep_time} );
 
     # default message
     $msg = $code >= 400 ? 'Failure' : 'Ok'
-        unless defined $msg;
+      unless defined $msg;
 
     # handle multiple lines
     my @lines;
 
-    if(ref $msg)
-    {
+    if ( ref $msg ) {
         confess "bad argument" unless ref $msg eq 'ARRAY';
         @lines = @$msg;
     }
-    else
-    {
-        @lines = split(/\r?\n/, $msg);
+    else {
+        @lines = split( /\r?\n/, $msg );
     }
-    for(my $i = 0; $i < @lines; $i++)
-    {
+    for ( my $i = 0 ; $i < @lines ; $i++ ) {
+
         # RFC says that all lines but the last must
         # split the code and the message with a dash (-)
         my $sep = $i == $#lines ? ' ' : '-';
@@ -634,21 +589,18 @@ sub reply
     }
 }
 
-sub get_hostname
-{
-    my($self) = @_;
+sub get_hostname {
+    my ($self) = @_;
     return HOSTNAME;
 }
 
-sub get_protoname
-{
-    my($self) = @_;
+sub get_protoname {
+    my ($self) = @_;
     return 'NOPROTO';
 }
 
-sub get_appname
-{
-    my($self) = @_;
+sub get_appname {
+    my ($self) = @_;
     return 'Net::Server::Mail (Perl)';
 }
 
@@ -671,29 +623,26 @@ Handler takes no argument.
 
 =cut
 
-sub banner
-{
-    my($self) = @_;
+sub banner {
+    my ($self) = @_;
 
-    unless(defined $self->{banner_string})
-    {
+    unless ( defined $self->{banner_string} ) {
         my $hostname  = $self->get_hostname  || '';
         my $protoname = $self->get_protoname || '';
         my $appname   = $self->get_appname   || '';
 
         my $str;
-        $str  = $hostname.' '  if length $hostname;
-        $str .= $protoname.' ' if length $protoname;
-        $str .= $appname.' '   if length $appname;
+        $str = $hostname . ' ' if length $hostname;
+        $str .= $protoname . ' ' if length $protoname;
+        $str .= $appname . ' '   if length $appname;
         $str .= 'Service ready';
         $self->{banner_string} = $str;
     }
 
-    $self->make_event
-    (
-        name => 'banner',
-        success_reply => [220, $self->{banner_string}],
-        failure_reply => ['',''],
+    $self->make_event(
+        name          => 'banner',
+        success_reply => [ 220, $self->{banner_string} ],
+        failure_reply => [ '', '' ],
     );
 }
 
@@ -701,24 +650,21 @@ sub banner
 
 =head2 timeout
 
-This event append where timeout is exeded.
+This event append where timeout is exceeded.
 
 Handler takes no argument.
 
 =cut
 
-sub timeout
-{
-    my($self) = @_;
+sub timeout {
+    my ($self) = @_;
 
-    $self->make_event
-    (
-        name => 'timeout',
-        success_reply => 
-        [
+    $self->make_event(
+        name          => 'timeout',
+        success_reply => [
             421,
-            $self->get_hostname . 
-                ' Timeout exceeded, closing transmission channel'
+            $self->get_hostname
+              . ' Timeout exceeded, closing transmission channel'
         ],
     );
 
@@ -727,6 +673,45 @@ sub timeout
 
 =pod
 
+=head2 timeout
+
+This event append where connection is closed or an error occurs during reading from socket.
+
+Takes the error description as an argument if an error occured and the argument is undefined if the session was closed by peer.
+
+    $mailserver->set_callback
+    (
+        'stop_session', sub
+        {
+            my($session, $err) = @_;
+            if( defined $err )
+            {
+                print "Error occured during processing: $err\n";
+            }
+            else
+            {
+                print "Session closed py peer\n";
+            }
+            return 1;
+        }
+    );
+
+=cut
+
+sub stop_session {
+    my ($self, $err) = @_;
+
+    $self->make_event(
+        name          => 'stop_session',
+	arguments     => [$err],
+	no_reply      => 1,
+    );
+
+    return 1;
+}
+
+=pod
+
 =head1 SEE ALSO
 
 Please, see L<Net::Server::Mail::SMTP>, L<Net::Server::Mail::ESMTP>
